@@ -16,7 +16,6 @@
 
 #include "tabletmodemanager_interface.h"
 
-#include <QElapsedTimer>
 #include <QGuiApplication>
 #include <QThread>
 #include <QTimer>
@@ -26,16 +25,13 @@
 
 #include <wayland-client-protocol.h>
 
-#include <chrono>
 #include <utility>
 
 using namespace KScreen;
-using namespace std::chrono_literals;
 
 WaylandConfig::WaylandConfig(QObject *parent)
     : QObject(parent)
     , m_outputManagement(std::make_unique<WaylandOutputManagement>(19))
-    , m_registryInitialized(false)
     , m_blockSignals(false)
     , m_kscreenConfig(new Config)
     , m_kscreenPendingConfig(nullptr)
@@ -51,7 +47,7 @@ WaylandConfig::WaylandConfig(QObject *parent)
 WaylandConfig::~WaylandConfig()
 {
     if (m_registry) {
-        wl_registry_destroy(m_registry);
+        destroyRegistry();
     }
 }
 
@@ -73,7 +69,7 @@ void WaylandConfig::initKWinTabletMode()
             return;
         }
         m_tabletModeEngaged = tabletMode;
-        if (!m_blockSignals && m_initializingOutputs.empty()) {
+        if (!m_blockSignals) {
             Q_EMIT configChanged();
         }
     });
@@ -82,7 +78,7 @@ void WaylandConfig::initKWinTabletMode()
             return;
         }
         m_tabletModeAvailable = available;
-        if (!m_blockSignals && m_initializingOutputs.empty()) {
+        if (!m_blockSignals) {
             Q_EMIT configChanged();
         }
     });
@@ -112,8 +108,10 @@ void WaylandConfig::setupRegistry()
 
     auto globalAdded = [](void *data, wl_registry *registry, uint32_t name, const char *interface, uint32_t version) {
         auto self = static_cast<WaylandConfig *>(data);
-        if (qstrcmp(interface, WaylandOutputDevice::interface()->name) == 0) {
+        if (qstrcmp(interface, kde_output_device_v2_interface.name) == 0) {
             self->addOutput(name, std::min(20u, version));
+        } else if (qstrcmp(interface, wl_fixes_interface.name) == 0) {
+            self->m_fixes = static_cast<wl_fixes *>(wl_registry_bind(registry, name, &wl_fixes_interface, 1));
         }
     };
 
@@ -126,23 +124,30 @@ void WaylandConfig::setupRegistry()
     static const wl_registry_listener registryListener{globalAdded, globalRemoved};
     wl_registry_add_listener(m_registry, &registryListener, this);
 
-    static const wl_callback_listener callbackListener{[](void *data, wl_callback *callback, uint32_t callbackData) {
-        Q_UNUSED(callback)
-        Q_UNUSED(callbackData)
-        auto self = static_cast<WaylandConfig *>(data);
-        self->m_registryInitialized = true;
-        self->checkInitialized();
-    }};
-    auto callback = wl_display_sync(waylandApp->display());
-    wl_callback_add_listener(callback, &callbackListener, this);
-    QElapsedTimer timer;
-    timer.start();
-    while (!m_initialized) {
-        if (timer.durationElapsed() >= 300ms) {
-            qCWarning(KSCREEN_WAYLAND) << "Connection to Wayland server timed out.";
-            break;
+    wl_display_roundtrip(display); // list output devices
+    wl_display_roundtrip(display); // get output device properties
+}
+
+void WaylandConfig::destroyRegistry()
+{
+    qDeleteAll(m_initializingOutputs);
+    m_initializingOutputs.clear();
+
+    auto outputs = std::move(m_outputMap);
+    m_screen->setOutputs({});
+    qDeleteAll(outputs);
+
+    if (m_registry) {
+        if (m_fixes) {
+            wl_fixes_destroy_registry(m_fixes, m_registry);
         }
-        wl_display_roundtrip(display);
+        wl_registry_destroy(m_registry);
+        m_registry = nullptr;
+    }
+
+    if (m_fixes) {
+        wl_fixes_destroy(m_fixes);
+        m_fixes = nullptr;
     }
 }
 
@@ -158,14 +163,8 @@ void WaylandConfig::handleActiveChanged()
     if (!m_registry) {
         return;
     }
-    qDeleteAll(m_initializingOutputs);
-    m_initializingOutputs.clear();
-    auto outputs = std::move(m_outputMap);
-    m_screen->setOutputs({});
-    qDeleteAll(outputs);
 
-    wl_registry_destroy(m_registry);
-    m_registry = nullptr;
+    destroyRegistry();
 
     if (!m_blockSignals) {
         Q_EMIT configChanged();
@@ -190,19 +189,11 @@ void WaylandConfig::addOutput(quint32 name, quint32 version)
     connect(device, &WaylandOutputDevice::done, this, [this, device]() {
         if (m_initializingOutputs.removeOne(device)) {
             m_outputMap.insert(device->id(), device);
-            checkInitialized();
+            m_screen->setOutputs(m_outputMap.values());
+        }
 
-            if (m_initializingOutputs.isEmpty()) {
-                m_screen->setOutputs(m_outputMap.values());
-            }
-            if (!m_blockSignals && m_initializingOutputs.isEmpty()) {
-                Q_EMIT configChanged();
-            }
-        } else {
-            // output got update must update current config
-            if (!m_blockSignals) {
-                Q_EMIT configChanged();
-            }
+        if (!m_blockSignals) {
+            Q_EMIT configChanged();
         }
     });
 
@@ -231,24 +222,9 @@ void WaylandConfig::removeOutput(WaylandOutputDevice *output)
     }
 }
 
-bool WaylandConfig::isReady() const
+bool WaylandConfig::isValid() const
 {
-    // clang-format off
-    return !m_blockSignals
-            && m_registryInitialized
-            && m_initializingOutputs.isEmpty()
-            && m_outputMap.count() > 0
-            && m_outputManagement->isActive();
-    // clang-format on
-}
-
-void WaylandConfig::checkInitialized()
-{
-    if (!m_initialized && isReady()) {
-        m_initialized = true;
-        m_screen->setOutputs(m_outputMap.values());
-        Q_EMIT initialized();
-    }
+    return m_outputManagement->isActive();
 }
 
 KScreen::ConfigPtr WaylandConfig::currentConfig()
